#include <math.h>
#include "camera.h"
#include "config.h"

extern int width, height;

Camera::Camera() {
	this->position = Vector3(0,0,0);
	this->target = Vector3(0,0,1);
	fov = M_PI/4;
}

Camera::Camera(const Vector3 &position, const Vector3 &target) {
	this->position = position;
	this->target = target;
	fov = M_PI/4;
}

void Camera::set_position(const Vector3 &position) {
	this->position = position;
}

void Camera::set_target(const Vector3 &target) {
	this->target = target;
}

void Camera::set_fov(double fov) {
	this->fov = fov;
}

Ray Camera::get_primary_ray(double x, double y) {
	Ray prim_ray;
	Vector3 dir;

	dir.x = x;
	dir.y = y; 
	dir.z = 1.0 / tan(fov / 2.0);

	Vector3 up(0,1,0); 
	Vector3 d = normalize(target-position);

	Vector3 i,j,k;
	
	i = cross(up, d);
	j = cross(d, i);
	k = d;

	Matrix4x4 m;

	m.matrix[0][0] = i.x; 
	m.matrix[1][0] = i.y;
	m.matrix[2][0] = i.z;

	m.matrix[0][1] = j.x;
	m.matrix[1][1] = j.y;
	m.matrix[2][1] = j.z;

	m.matrix[0][2] = k.x;
	m.matrix[1][2] = k.y;
	m.matrix[2][2] = k.z;

	prim_ray.origin = position;
	dir.transform(m);
	prim_ray.dir = dir * RAY_MAG;
	return prim_ray;
}

