#ifndef MESH_H_
#define MESH_H_

#include <vector>
#include "object.h"
#include "vector.h"
#include "bbox.h"

enum MeshPrim {
	MESH_PRIM_TRI = 3,
	MESH_PRIM_QUAD = 4
};

struct Vertex {
	Vector3 pos;
	Vector3 norm;
	// Vector3 tex;
};

class Face {
public:
	Vertex v[4];
	Vector3 norm;	// face normal

	void calc_normal();
	Vector3 sample(MeshPrim prim) const;
};

class Mesh : public Object {
protected:
	MeshPrim prim;
	std::vector<Face> faces;

	bool (*face_intersection)(const Face*, const Ray&, IntInfo*);

public:

	Mesh(MeshPrim prim = MESH_PRIM_TRI);
	virtual ~Mesh();

	void set_primitive(MeshPrim prim);
	MeshPrim get_primitive() const;

	void add_face(const Face &face);
	int get_face_count() const;
	Face *get_face(int idx);
	const Face *get_face(int idx) const;

	virtual bool intersection(const Ray &ray, IntInfo *i_info) const;
	virtual void calc_bbox();
	virtual Vector3 sample() const;
};

#endif
